'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Array_iterator = [][Symbol.iterator];

/**
 * String template
 */

var Template = function () {
    /**
     * @param {string}          raw
     * @param {stirng[]}        [varName]  - Name list of the Local variable
     * @param {ChangedCallback} [onChange] - Call with New & Old value
     * @param {Array}           [bindData] - The parameter bound to `onChange`
     */
    function Template(raw, varName, onChange, bindData) {
        _classCallCheck(this, Template);

        this.length = 0;

        this.raw = raw, this.value = null;

        if (varName instanceof Function) bindData = onChange, onChange = varName, varName = null;

        this.varName = varName || [], this.expression = [];

        this.context = new Set(this.varName.concat('this'));

        this.onChange = onChange instanceof Function ? onChange : null;

        this.data = bindData || [];

        this.parse().clear();
    }

    _createClass(Template, [{
        key: Symbol.iterator,
        value: function value() {
            return Array_iterator.call(this);
        }

        /**
         * @type {RegExp}
         */

    }, {
        key: 'push',
        value: function push() {
            return Array.prototype.push.apply(this, arguments);
        }

        /**
         * @private
         *
         * @param {string} expression
         *
         * @return {number} Index of this Evaluation function
         */

    }, {
        key: 'compile',
        value: function compile(expression) {

            return this.expression.push(new (Function.prototype.bind.apply(Function, [null].concat(_toConsumableArray(this.varName), ['return ' + expression.trim()])))()) - 1;
        }

        /**
         * @private
         *
         * @return {Template}
         */

    }, {
        key: 'parse',
        value: function parse() {
            var _this = this;

            var addReference = function addReference(match, context, key1, key2, key3) {

                if (_this.context.has(context)) _this.push(key1 || key2 || key3);
            };

            this.raw = this.raw.replace(Template.Expression, function (_, expression) {

                expression.replace(Template.Reference, addReference);

                return '${' + _this.compile(expression) + '}';
            });

            return this;
        }

        /**
         * Evaluate expression
         *
         * @param {?object} context     - Value of `this` in the expression
         * @param {...*}    [parameter] - One or more value of the Local variable
         *
         * @return {string}
         */

    }, {
        key: 'evaluate',
        value: function evaluate(context) {
            var _this2 = this;

            for (var _len = arguments.length, parameter = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
                parameter[_key - 1] = arguments[_key];
            }

            var value = this.raw.replace(/\$\{(\d+)\}/g, function (_, index) {
                return _this2.expression[index].apply(context, parameter);
            });

            if (value !== this.value) {
                /**
                 * Call back only on Value changed
                 *
                 * @typedef {function} ChangedCallback
                 *
                 * @param {*}    newValue
                 * @param {*}    oldValue
                 * @param {...*} bindData
                 */
                if (this.onChange) this.onChange.apply(this, _toConsumableArray([value, this.value].concat(this.data)));

                this.value = value;
            }

            return value;
        }

        /**
         * @return {string} Value evaluated with empty data
         */

    }, {
        key: 'clear',
        value: function clear() {

            var data = {};

            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = this[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var key = _step.value;
                    data[key] = '';
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }

            return this.evaluate.apply(this, _toConsumableArray(Array.from(this.context, function () {
                return data;
            })));
        }
    }, {
        key: 'toString',
        value: function toString() {

            return this.value + '';
        }
    }], [{
        key: 'Expression',
        get: function get() {
            return (/\$\{([\s\S]+?)\}/g
            );
        }

        /**
         * @type {RegExp}
         */

    }, {
        key: 'Reference',
        get: function get() {
            return (/(\w+)(?:\.(\w+)|\[(?:'([^']+)|"([^"]+)))/g
            );
        }
    }]);

    return Template;
}();

exports.default = Template;