import View from './View';

import ObjectView from './ObjectView';

const Array_iterator = [ ][Symbol.iterator];


/**
 * View for Array model
 */
export default  class ArrayView extends View {
    /**
     * @param {Element} element
     */
    constructor(element) {

        if (! super(element,  'array',  [ ]).booted) {

            this.template = element.innerHTML.trim();  this.clear();
        }
    }

    [Symbol.iterator]() {  return  Array_iterator.call( this );  }

    clear() {

        Array.prototype.splice.call(this, 0, Infinity);

        this.content.innerHTML = '';

        return this;
    }

    valueOf() {  return  Array.from(this,  view => view.valueOf());  }

    /**
     * @param {Iterable} list
     *
     * @return {ArrayView}
     */
    render(list) {

        const data = this.data;

        this.content.append(... [ ].concat(
            ... Array.from(list,  item => {

                const view = this[ this.length++ ] = new ObjectView( this.template );

                data[ data.length ] = view.data;

                return  view.render( item ).content;
            })
        ));

        return this;
    }

    push(... item) {

        return  this.render( item ).length;
    }
}
